using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Collections.Generic;
using System.Linq;

namespace SemiColinGames {
  enum Terrain {
    Empty,
    Grass,
    GrassL,
    GrassR,
    Rock,
    RockL,
    RockR,
    Water,
    Block
  }

  class Tile {
    readonly Texture2D texture;

    public Tile(Texture2D texture, Terrain terrain, Rectangle position) {
      this.texture = texture;
      Terrain = terrain;
      Position = position;
    }

    public Rectangle Position { get; private set; }
    public Terrain Terrain { get; private set; }

    public void Draw(SpriteBatch spriteBatch, Camera camera) {
      int size = World.TileSize;
      Vector2 drawPos = new Vector2(Position.Left - camera.Left, Position.Top);
      switch (Terrain) {
        case Terrain.Grass: {
          Rectangle source = new Rectangle(3 * size, 0 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.GrassL: {
          Rectangle source = new Rectangle(2 * size, 0 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.GrassR: {
          Rectangle source = new Rectangle(4 * size, 0 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.Rock: {
          Rectangle source = new Rectangle(3 * size, 1 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.RockL: {
          Rectangle source = new Rectangle(1 * size, 2 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.RockR: {
          Rectangle source = new Rectangle(5 * size, 2 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.Water: {
          Rectangle source = new Rectangle(9 * size, 2 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;
        }
        case Terrain.Block: {
          Rectangle source = new Rectangle(6 * size, 3 * size, size, size);
          spriteBatch.Draw(texture, drawPos, source, Color.White);
          break;    
        }
        case Terrain.Empty:
        default:
          break;
      }
    }
  }

  class World {

    public const int TileSize = 16;
    readonly Tile[] tiles;
    readonly Rectangle[] collisionTargets;

    public int Width { get; private set; }
    public int Height { get; private set; }

    readonly string worldString = @"
                                                                                                                                                                                                                  
                                                                                                                                                                                                                  
                                                                                                                                                                                                                  
                                                                                                                                                                                                                  
                                                                                                                                                                                                     X            
                                                                                                                                                                                                     .            
                     X                                                         <======>   <==X              X           <=>    <XX>                                                        XX        .            
                                                                                                                                                                                          XXX        .            
                                                                                                                                                                                         XXXX        .            
                                                                                                                                                                                        XXXXX        .            
               X   <X=X>                     <>         <>                  <X>              =     <>    X  X  X     X          <>      X  X          XX  X            <=X>            XXXXXX        .            
                                     <>      []         []                                                                             XX  XX        XXX  XX                          XXXXXXX        .            
                           <>        []      []         []                                                                            XXX  XXX      XXXX  XXX     <>              <> XXXXXXXX                     
                           []12345678[]123456[]123456789[]1234567890  123456   123456         12345  1234       12345      1234   123XXXX  XXXX1234XXXXX  XXXX1234[]123    1234567[]XXXXXXXXX12345678             
====================> <====..========..======..=========..=========>  <=============>   <==============================================================>  <=======..==============..==============================
....................] [............................................]  [.............]   [..............................................................]  [.......................................................";

    public World(Texture2D texture) {
      var tilesList = new List<Tile>();
      string[] worldDesc = worldString.Split('\n');
      Width = worldDesc.AsQueryable().Max(a => a.Length);
      Height = worldDesc.Length;
      Debug.WriteLine("world size: {0}x{1}", Width, Height);
      for (int i = 0; i < Width; i++) {
        for (int j = 0; j < Height; j++) {
          Terrain terrain = Terrain.Empty;
          if (i < worldDesc[j].Length) {
            switch (worldDesc[j][i]) {
              case '=':
                terrain = Terrain.Grass;
                break;
              case '<':
                terrain = Terrain.GrassL;
                break;
              case '>':
                terrain = Terrain.GrassR;
                break;
              case '.':
                terrain = Terrain.Rock;
                break;
              case '[':
                terrain = Terrain.RockL;
                break;
              case ']':
                terrain = Terrain.RockR;
                break;
              case '~':
                terrain = Terrain.Water;
                break;
              case 'X':
                terrain = Terrain.Block;
                break;
              default:
                terrain = Terrain.Empty;
                break;
            }
          }
          if (terrain != Terrain.Empty) {
            var position = new Rectangle(i * TileSize, j * TileSize, TileSize, TileSize);
            tilesList.Add(new Tile(texture, terrain, position));
          }
        }
      }
      tiles = tilesList.ToArray();
      collisionTargets = new Rectangle[tiles.Length];
      for (int i = 0; i < tiles.Length; i++) {
        collisionTargets[i] = tiles[i].Position;
      }
    }

    public void Draw(SpriteBatch spriteBatch, Camera camera) {
      foreach (Tile t in tiles) {
        t.Draw(spriteBatch, camera);
      }
    }

    public Rectangle[] CollisionTargets {
      get { return collisionTargets; }
    }
  }
}
